/* Program name --- naive.c
 *       written by Sakurai   2005.12.6
 */

#include        <stdio.h>
#include        <string.h>
#include        <stdlib.h>
#include        <math.h>
#include        <time.h>
#include        "search.h"
#include        "submatch.h"

void cubic_disjoint();
void malloc_read();
void error(char *);

struct  SeqData **sqd, qsqd[1];
dp_match_t *dpm;

main(argc,argv)
int	argc;
char	*argv[];
{
  int        sid, size, option=NO;
  char       dbfile[BUFSIZE], seqfile[BUFSIZE], queryfile[BUFSIZE];
  clock_t    timecnt;
  double     totaltime=0.0;
  float epsilon;

  if (argc != 1)
    if ( strcmp(argv[1],"-a") == 0 ) 
      option = YES;

  if(argc != 4 + option)
  {
    error(argv[0]);
    exit(1);
  }

  /*
   * Get the file names
   */
  strcpy(dbfile,argv[1+option]);
  strcpy(queryfile,argv[2+option]);
  epsilon = atof(argv[3+option]);

  /*
   * Memory allocation and read data
   */
  malloc_read(dbfile, option, queryfile, &size);

  /*
   * exact DTW
   */
  system("/bin/date");
  timecnt = clock();

  /* 
   * Extract subsequences whose distances are less than Epsilon
   */
  cubic_disjoint(size, epsilon);

  totaltime = (double)(clock() - timecnt) / (double)CLOCKS_PER_SEC;
  system("/bin/date");
  printf("%.4f sec.\n",totaltime);

  subMatch_clear(dpm);
  free(buf_float);
  free(sqd[0]);
  free(sqd);
}

void cubic_disjoint(size, epsilon)
int size;
float epsilon;
{
  int i, sid;

  for (sid=0 ; sid<size ; sid++){
    subMatch_set_sequence_length(dpm, sqd[sid]->length, qsqd->length);
    subMatch_cubic_disjoint(dpm, sqd[sid]->value, qsqd->value, epsilon);
#if RESULT
    printf("[sid:%d]\n", sid);
    for (i=0 ; i<dpm->subseqs ; i++)
      printf("%d. start:%d, length:%d, distance:%f\n", i, 
	   dpm->ans[i].start_point, dpm->ans[i].length, dpm->ans[i].distance);
#endif
  }
}

void malloc_read(dbfile, option, queryfile, size)
char *dbfile;
int  option;
char *queryfile;
int  *size;
{

  int     i, j, s;
  int     maxlen, sid;
  char    seqfile[BUFSIZE];
  struct  SeqData *buf_sqd;
  int     sqd_cnt = 0;
  FILE    *fp;

  if (option == YES)
    s = count_lines(dbfile);
  else
    s = 1;
  sqd = (struct SeqData **)malloc(sizeof(struct SeqData *) * s);
  buf_sqd = (struct SeqData *)malloc(sizeof(struct SeqData) * s);
  if (sqd == NULL || buf_sqd == NULL){
    printf("Allocation error : sqd\n");
    exit(1);
  }
  sqd_cnt = 0;
  for (sid=0 ; sid<s ; sid++){
    sqd[sid] = buf_sqd + sqd_cnt;
    sqd_cnt++;
  }

  qsqd->length = count_lines(queryfile);

  /*
   * read dbfile and queryfile
   */
  if (option == YES){
    if ( ( fp = fopen ( dbfile , "r" ) ) == NULL ){
      printf( "  %s can not open\n" , dbfile);
      exit(1);
    }
    maxlen = 0;
    for (sid=0 ; sid<s ; sid++){
      fscanf(fp, "%s\n", seqfile);
      sqd[sid]->length = count_lines(seqfile);
      if (maxlen < sqd[sid]->length)
	maxlen = sqd[sid]->length;
    }
    rewind(fp);
  }
  else
    maxlen = sqd[0]->length = count_lines(dbfile);

  j = 0;
  /* data sequences */
  for (sid=0 ; sid<s ; sid++)
    j += sqd[sid]->length;
  /* query sequence */
  j += qsqd->length;

  malloc_float(j);

  for (sid=0 ; sid<s ; sid++)
    sqd[sid]->value = allot_float(sqd[sid]->length);
  qsqd->value = allot_float(qsqd->length);

  dpm = subMatch_malloc(maxlen, qsqd->length);

  /*
   * Read data
   */
  if (option == YES){
    for (sid=0 ; sid<s ; sid++){
      fscanf(fp, "%s\n", seqfile);
      read_file(seqfile, sqd[sid]->value, sqd[sid]->length);
    }
    fclose(fp);
  }
  else
    read_file(dbfile, sqd[0]->value, sqd[0]->length);
  read_file(queryfile, qsqd->value, qsqd->length);

  *size = s;
}

void error(char *toolname)
{
  fprintf(stderr,"Usage : %s  [-a]  dbfile  queryfile  epsilon\n",toolname);
  exit(1);
}
