/******************************************************************************
* File name     : The definition of Subsequence Matching Module
* Date          : December 6, 2005
* Note          : Definition of Macros, tables, function-like macros, functions
******************************************************************************/

#include <limits.h>

  /*--------------------------------------------------------------------------*
   * Marco definition
   *--------------------------------------------------------------------------*
   */
#define   INITIALIZE_VALUE  1e20   /* Initial distance value                 */
#define   DIAGONAL_WEIGHT      1
#define   OPTIMAL              1
#define   RESULT               1

  /*--------------------------------------------------------------------------*
   * Table definition
   *--------------------------------------------------------------------------*
   */

  /*
   * Management table keeps the intermediate results of distance computation 
   * for dynamic programming
   */
typedef struct _dp_match {
  struct _dp_match_opt_value *result1;
  struct _dp_match_opt_value *result2;
  int    seq1_length;
  int    seq2_length;
  int    subseqs;              // Used only for the naive method 
  struct _opt_subseq *ans;     // Used only for the naive method 
} dp_match_t;

  /*
   * Optimum distance value during processing
   */
struct _dp_match_opt_value {
  float optimal_value;
  int   column_number;
};

  /*
   * Buffer for keeping query results
   */
struct _opt_subseq {
  float distance;
  int   start_point;
  int   length;
};

  /*--------------------------------------------------------------------------*
   * Definition of function-like macros
   *--------------------------------------------------------------------------*
   */

  /*
   * For computing the distance at the current grid point
   */
#define CALC_DISTANCE(SP1, SP2) \
  ((SP1 - SP2) * (SP1 - SP2))

  /*
   * For setting the sequence lengths
   */
#define subMatch_set_sequence_length(self, seq1, seq2) \
  (self)->seq1_length = seq1; \
  (self)->seq2_length = seq2

  /*--------------------------------------------------------------------------*
   * Functions
   *--------------------------------------------------------------------------*
   */

dp_match_t* subMatch_malloc(
  size_t      buffer_size1,
  size_t      buffer_size2
);
void subMatch_clear(
  dp_match_t* self
);
void subMatch_pad_disjoint(
  dp_match_t* self,
  float      *sequence1,
  float      *sequence2,
  float      epsilon
);
void subMatch_cubic_disjoint(
  dp_match_t* self,
  float      *sequence1,
  float      *sequence2,
  float      epsilon
);
